#include "fontx.h"

////// FONTX形式のフォントファイルからフォントファイルを読み込み。 //////
// 対応フォントはFONTX形式　最小8ドット四方・最大255ドット四方まで対応。
// 最小フォントサイズは実用値、最大フォントサイズは論理値。
//
// 受け渡し用の変数：
//   code  ： 文字コード（符号なしint型）
//   *pat  ： 戻ってきた文字パターンを格納するための配列変数（符号なしchar型）
//            1フォントあたりのバイト数以上を確保すること。
//
// 戻り値： 符号なしlong型。
 
unsigned long KanjiReadX(unsigned int code , unsigned char *pat) {
 
  // 参考資料：http://www.elm-chan.org/docs/dosv/fontx.html
 
  // オフセット計算用
  // ccc 　= 現在の文字数(Character Count C…？)
  // ofset = 最終的なオフセット値（OFfSET）
  unsigned long ccc = 0 , ofset = 0;
 
  // フォントファイルを開く
  // 文字コードを判別して、文字コードが1バイトの範囲内なら半角コード
  // 2バイトの範囲内なら全角コード用のファイルを読み込む。
  File fontx_file;
  if (code < 0x100)  fontx_file = SD.open(FONT_FILEA, FILE_READ);
  if (code > 0x100)  fontx_file = SD.open(FONT_FILEJ, FILE_READ);
 
  // フォントファイルのファイルタイプとファイル名は不要なので
  // その分ファイルをシークしておく
  fontx_file.seek(14);
 
  // フォントデータの読み込み。
  // Fw = 文字サイズ（横）単位：ピクセル
  // Fh = 文字サイズ（縦）単位：ピクセル
  // Ft = 文字コード（00ならアスキー 01ならShift_JIS）最後2つは2バイト文字のみの変数
  // Fb = コードブロック数（Fbwはループ用）
  uint8_t Fw = fontx_file.read() , Fh  = fontx_file.read(), Ft  = fontx_file.read() , Fb, Fbw;
 
  // 1文字あたりのデータサイズ（バイト）を計算
  uint16_t Fsize = ((Fw + 7) / 8 ) * Fh ;
 
  // コードフラグが0なら英文フォント用のオフセットを書き込む。
  if (Ft == 0x00) {
    // ヘッダサイズ ＋ 文字コード×1文字あたりのサイズ
    if (code < 0x100) ofset = 17 + code * Fsize ;
 
  } else {
    // ブロック数のデータは2バイト用フォントにしか無いのでココで取得する。
    Fb = Fbw = fontx_file.read();
    // フォントオフセット取得用ループ
    while (Fbw --) {
 
      // コードブロックからコードの範囲を読み込む
      // Cbs = 開始コード（Code Block Start）
      // Cbe = 終了コード（Code Block End）
      uint16_t Cbs = fontx_file.read() | fontx_file.read() << 8 , Cbe = fontx_file.read() | fontx_file.read() << 8;
 
      // 指示された文字コードがコードブロック内に存在するかどうかを確認
      if (code >= Cbs && code <= Cbe) {
 
        // コードブロック内に文字が存在したのなら
        // そのコードブロック内で何文字目に存在するのかを取得
        ccc += code - Cbs;
 
        // 全体のオフセット値を計算する
        // ヘッダサイズ＋4バイト×コードブロック数＋直前の文字コードまでの文字数×1文字あたりのサイズ
        ofset = 18 + 4 * Fb + ccc * Fsize;
 
        // 抜ける
        break;
      }
 
      // 取得したブロック内に該当文字コードが存在しない場合。
      // 存在しなかったブロック内の文字数を計算して次へ。
      ccc += Cbe - Cbs + 1 ;
    }
 
  }
 
  // ループ用にフォントサイズを退避
  uint16_t i = Fsize;
 
  // 指示された文字コードが見つかった場合の処理
  if (ofset) {
    fontx_file.seek(ofset);
 
    // パターン格納用の変数に漢字パターンを取得し、格納
    while (i--) *(pat++) = fontx_file.read();
    // 終わったらファイルを閉じる
    fontx_file.close();
 
    // フォントデータをリターンして終了
    return ((unsigned long)Fw << 24) | ((unsigned long)Fh << 16) | Fsize;
 
    // 戻り値は4バイトデータで、1バイトごとに上から
    //  1バイト目   ＝フォントサイズ（幅）
    //  2バイト目   ＝フォントサイズ（高さ）
    //  3・4バイト目＝フォントデータのバイト数（2バイト）
    // を返す。
 
  }
 
 
  // 見つからなかった場合はファイルを閉じて0をリターンして終了
  fontx_file.close();
  return 0 ;
}



// Shift_JIS用全角→半角変換関数
// 入力されたASCIIコードを元に該当する全角文字のShiftJISコードを返す。
// 例えば「A」だと「Ａ」の文字コードが返ってくる
int AsciiZen(char as_code){
	unsigned int jis_code = 0;
	
	// 文字コード変換
	
	// 記号はShiftJISの方がASCIIと対になっていないので
	// switch命令で切り替えて固定値を返す。
	// 英数字は部分的に連続しているため、計算してその結果を返す
	
    switch (as_code){
		
		////// 記号ここから //////
		
				case 0x20 :jis_code = 0x8140;
				break;
				case 0x21 :jis_code = 0x8149;
				break;
				case 0x22 :jis_code = 0x818D;
				break;
				case 0x23 :jis_code = 0x8194;
				break;
				case 0x24 :jis_code = 0x8190;
				break;
				case 0x25 :jis_code = 0x8193;
				break;
				case 0x26 :jis_code = 0x8195;
				break;
				case 0x27 :jis_code = 0x8166;
				break;
				case 0x28 :jis_code = 0x8169;
				break;
				case 0x29 :jis_code = 0x816A;
				break;
				case 0x2A :jis_code = 0x8196;
				break;
				case 0x2B :jis_code = 0x817B;
				break;
				case 0x2C :jis_code = 0x8143;
				break;
				case 0x2D :jis_code = 0x817C;
				break;
				case 0x2E :jis_code = 0x8144;
				break;
				case 0x2F :jis_code = 0x815E;
				break;
				case 0x3A :jis_code = 0x8146;
				break;
				case 0x3B :jis_code = 0x8147;
				break;
				case 0x3C :jis_code = 0x8183;
				break;
				case 0x3D :jis_code = 0x8181;
				break;
				case 0x3E :jis_code = 0x8184;
				break;
				case 0x3F :jis_code = 0x8148;
				break;
				case 0x40 :jis_code = 0x8197;
				break;
				case 0x5B :jis_code = 0x816D;
				break;
				case 0x5C :jis_code = 0x818F;
				break;
				case 0x5D :jis_code = 0x816E;
				break;
				case 0x5E :jis_code = 0x814F;
				break;
				case 0x5F :jis_code = 0x8151;
				break;
				case 0x60 :jis_code = 0x814D;
				break;
				case 0x7B :jis_code = 0x816F;
				break;
				case 0x7C :jis_code = 0x8162;
				break;
				case 0x7D :jis_code = 0x8170;
				break;
				case 0x7E :jis_code = 0x8160;
				break;
				
		///// 記号ここまで /////
				
		///// 以下英数字の変換 //////
				default :
				
				if (as_code >= 0x30 && as_code <= 0x39 ) jis_code = 0x821F + as_code ;
				if (as_code >= 0x41 && as_code <= 0x5A ) jis_code = 0x821F + as_code ;
				if (as_code >= 0x60 && as_code <= 0x7A ) jis_code = 0x8220 + as_code ;
				
				break;
		}

		// シフトJISコードを返す。
		return jis_code;


	}